@@ -2,12 +2,14 @@ import warnings
 from io import IOBase
 from typing import *
 import platform
+import os
 
 import requests
 from lightly.api.api_workflow_tags import _TagsMixin
 from requests import Response
 
 from lightly.__init__ import __version__
+from lightly.api.api_workflow_collaboration import _CollaborationMixin
 from lightly.api.api_workflow_compute_worker import _ComputeWorkerMixin
 from lightly.api.api_workflow_datasets import _DatasetsMixin
 from lightly.api.api_workflow_datasources import _DatasourcesMixin
@@ -17,12 +19,12 @@ from lightly.api.api_workflow_upload_dataset import _UploadDatasetMixin
 from lightly.api.api_workflow_upload_embeddings import _UploadEmbeddingsMixin
 from lightly.api.api_workflow_upload_metadata import _UploadCustomMetadataMixin
 from lightly.api.bitmask import BitMask
-from lightly.api.utils import getenv
+from lightly.api.utils import DatasourceType, get_signed_url_destination, getenv
 from lightly.api.version_checking import get_minimum_compatible_version, \
     version_compare
-from lightly.openapi_generated.swagger_client import TagData, ScoresApi, \
-    QuotaApi, TagArithmeticsRequest, TagArithmeticsOperation, \
-    TagBitMaskResponse
+from lightly.openapi_generated.swagger_client.api.collaboration_api import CollaborationApi
+from lightly.openapi_generated.swagger_client import ScoresApi, \
+    QuotaApi, MetaDataConfigurationsApi
 from lightly.openapi_generated.swagger_client.api.datasets_api import \
     DatasetsApi
 from lightly.openapi_generated.swagger_client.api.datasources_api import \
@@ -44,6 +46,8 @@ from lightly.openapi_generated.swagger_client.models.dataset_data import \
     DatasetData
 from lightly.utils.reordering import sort_items_by_keys
 
+# Env variable for server side encryption on S3
+LIGHTLY_S3_SSE_KMS_KEY = 'LIGHTLY_S3_SSE_KMS_KEY' 
 
 class ApiWorkflowClient(_UploadEmbeddingsMixin,
                         _SelectionMixin,
@@ -54,6 +58,7 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
                         _TagsMixin,
                         _DatasourcesMixin,
                         _ComputeWorkerMixin,
+                        _CollaborationMixin,
                         ):
     """Provides a uniform interface to communicate with the api 
     
@@ -90,6 +95,7 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
         api_client = ApiClient(configuration=configuration)
         self.user_agent = f"Lightly/{__version__}/python ({platform.platform()})"
         self.api_client = api_client
+        self.set_request_timeout(DEFAULT_API_TIMEOUT)
 
         self.token = token
         if dataset_id is not None:
@@ -97,6 +103,7 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
         if embedding_id is not None:
             self.embedding_id = embedding_id
 
+        self._collaboration_api = CollaborationApi(api_client=self.api_client)
         self._compute_worker_api = DockerApi(api_client=self.api_client)
         self._datasets_api = DatasetsApi(api_client=self.api_client)
         self._datasources_api = DatasourcesApi(api_client=self.api_client)
@@ -108,6 +115,8 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
         self._scores_api = ScoresApi(api_client=api_client)
         self._samples_api = SamplesApi(api_client=api_client)
         self._quota_api = QuotaApi(api_client=api_client)
+        self._metadata_configurations_api = \
+            MetaDataConfigurationsApi(api_client=api_client)
 
     def check_version_compatibility(self):
         minimum_version = get_minimum_compatible_version()
@@ -188,10 +197,13 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
             get_sample_mappings_by_dataset_id(dataset_id=self.dataset_id, field="fileName")
         return filenames_on_server
 
-    def upload_file_with_signed_url(self,
-                                    file: IOBase,
-                                    signed_write_url: str,
-                                    headers: Dict = None) -> Response:
+    def upload_file_with_signed_url(
+        self,
+        file: IOBase,
+        signed_write_url: str,
+        headers: Optional[Dict] = None,
+        session: Optional[requests.Session] = None,
+    ) -> Response:
         """Uploads a file to a url via a put request.
 
         Args:
@@ -202,19 +214,83 @@ class ApiWorkflowClient(_UploadEmbeddingsMixin,
                 the url must be a signed write url.
             headers:
                 Specific headers for the request.
+            session:
+                Optional requests session used to upload the file.
 
         Returns:
             The response of the put request, usually a 200 for the success case.
 
         """
+
+        # check to see if server side encryption for S3 is desired
+        # see https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingServerSideEncryption.html
+        # see https://docs.aws.amazon.com/AmazonS3/latest/userguide/UsingKMSEncryption.html
+        lightly_s3_sse_kms_key = os.environ.get(LIGHTLY_S3_SSE_KMS_KEY, '').strip()
+        # Only set s3 related headers when we are talking with s3
+        if get_signed_url_destination(signed_write_url)==DatasourceType.S3 and lightly_s3_sse_kms_key:
+            if headers is None:
+                headers = {}
+            # don't override previously set SSE
+            if 'x-amz-server-side-encryption' not in headers:
+                if lightly_s3_sse_kms_key.lower() == 'true':
+                    # enable SSE with the key of amazon
+                    headers['x-amz-server-side-encryption'] = 'AES256'
+                else:
+                    # enable SSE with specific customer KMS key
+                    headers['x-amz-server-side-encryption'] = 'aws:kms'
+                    headers['x-amz-server-side-encryption-aws-kms-key-id'] = lightly_s3_sse_kms_key
+
+        # start requests session and make put request
+        sess = session or requests
         if headers is not None:
-            response = requests.put(signed_write_url, data=file, headers=headers)
+            response = sess.put(signed_write_url, data=file, headers=headers)
         else:
-            response = requests.put(signed_write_url, data=file)
-
-        if response.status_code < 200 or response.status_code >= 300:
-            msg = f'Failed PUT request to {signed_write_url} with status_code'
-            msg += f'{response.status_code}!'
-            raise RuntimeError(msg)
-
+            response = sess.put(signed_write_url, data=file)
+        response.raise_for_status()
         return response
+
+    def set_request_timeout(self, timeout: Union[int, Tuple[int, int]]):
+        """Sets a default timeout for all api requests.
+
+        Args:
+            timeout:
+                Timeout in seconds. Is either a single total_timeout value or a
+                (connect_timeout, read_timeout) tuple. 
+                See https://urllib3.readthedocs.io/en/stable/reference/urllib3.util.html?highlight=timeout#urllib3.util.Timeout
+                for details on the different values.
+        """
+        set_api_client_request_timeout(client=self.api_client, timeout=timeout)
+
+
+DEFAULT_API_TIMEOUT = 60 * 3 # seconds
+
+def set_api_client_request_timeout(
+    client: ApiClient, 
+    timeout: Union[int, Tuple[int, int]] = DEFAULT_API_TIMEOUT,
+):
+    """Sets a default timeout for all requests with the client.
+
+    This function patches the request method of the api client. This is 
+    necessary because the swagger api client does not respect any timeouts 
+    configured by urllib3. Instead it expects a timeout to be passed with every
+    request. Code here: https://github.com/lightly-ai/lightly/blob/ffbd32fe82f76b37c8ac497640355314474bfc3b/lightly/openapi_generated/swagger_client/rest.py#L141-L148
+
+    Args:
+        client:
+            Api client on which the timeout is applied.
+        timeout:
+            Timeout in seconds. Is either a single total_timeout value or a
+            (connect_timeout, read_timeout) tuple. 
+            See https://urllib3.readthedocs.io/en/stable/reference/urllib3.util.html?highlight=timeout#urllib3.util.Timeout
+            for details on the different values.
+
+    """
+    request_fn = client.rest_client.request
+
+    def new_request_fn(*args, **kwargs):
+        request_timeout = kwargs['_request_timeout']
+        if request_timeout is None:
+            kwargs['_request_timeout'] = timeout
+        return request_fn(*args, **kwargs)
+
+    client.rest_client.request = new_request_fn
