@@ -6,13 +6,14 @@
 import torch
 import torch.nn as nn
 
-from typing import List
+from typing import Dict, List, Optional, Tuple, Union
 
 from PIL import Image
 import torchvision
 import torchvision.transforms as T
 
 from lightly.transforms import GaussianBlur
+from lightly.transforms import Jigsaw
 from lightly.transforms import RandomRotate
 from lightly.transforms import RandomSolarization
 
@@ -21,6 +22,7 @@ imagenet_normalize = {
     'std': [0.229, 0.224, 0.225]
 }
 
+
 class BaseCollateFunction(nn.Module):
     """Base class for other collate implementations.
 
@@ -645,3 +647,250 @@ class DINOCollateFunction(MultiViewCollateFunction):
         transforms = [global_transform_0, global_transform_1]
         transforms.extend(local_transforms)
         super().__init__(transforms)
+
+
+class MAECollateFunction(MultiViewCollateFunction):
+    """Implements the view augmentation for MAE [0].
+
+    - [0]: Masked Autoencoder, 2021, https://arxiv.org/abs/2111.06377
+
+    Attributes:
+        input_size:
+            Size of the input image in pixels.
+        min_scale:
+            Minimum size of the randomized crop relative to the input_size.
+        normalize:
+            Dictionary with 'mean' and 'std' for torchvision.transforms.Normalize.
+
+     """
+    def __init__(
+        self,
+        input_size: Union[int, Tuple[int, int]] = 224,
+        min_scale: float = 0.2,
+        normalize: dict = imagenet_normalize,
+    ):
+        transforms = [
+            T.RandomResizedCrop(input_size, scale=(min_scale, 1.0), interpolation=3),  # 3 is bicubic
+            T.RandomHorizontalFlip(),
+            T.ToTensor(),
+        ]
+
+        if normalize:
+            transforms.append(
+                T.Normalize(
+                    mean=normalize['mean'],
+                    std=normalize['std']
+                )
+            )
+        super().__init__([T.Compose(transforms)])
+
+    def forward(self, batch: List[tuple]):
+        views, labels, fnames = super().forward(batch)
+        # Return only first view as MAE needs only a single view per image.
+        return views[0], labels, fnames
+
+class PIRLCollateFunction(nn.Module):
+    """Implements the transformations for PIRL [0]. The jigsaw augmentation
+    is applied during the forward pass.
+
+    - [0] PIRL, 2019: https://arxiv.org/abs/1912.01991
+
+    Attributes:
+        input_size:
+            Size of the input image in pixels.
+        cj_prob:
+            Probability that color jitter is applied.
+        cj_bright:
+            How much to jitter brightness.
+        cj_contrast:
+            How much to jitter constrast.
+        cj_sat:
+            How much to jitter saturation.
+        cj_hue:
+            How much to jitter hue.
+        min_scale:
+            Minimum size of the randomized crop relative to the input_size.
+        random_gray_scale:
+            Probability of conversion to grayscale.
+        hf_prob:
+            Probability that horizontal flip is applied.
+        n_grid:
+            Sqrt of the number of grids in the jigsaw image.
+        normalize:
+            Dictionary with 'mean' and 'std' for torchvision.transforms.Normalize.
+
+    Examples:
+
+        >>> # PIRL for ImageNet
+        >>> collate_fn = PIRLCollateFunction()
+        >>> 
+        >>> # PIRL for CIFAR-10
+        >>> collate_fn = PIRLCollateFunction(
+        >>>     input_size=32,
+        >>> )
+
+    """
+
+    def __init__(self,
+                 input_size: int = 64,
+                 cj_prob: float = 0.8,
+                 cj_bright: float = 0.4,
+                 cj_contrast: float = 0.4,
+                 cj_sat: float = 0.4,
+                 cj_hue: float = 0.4,
+                 min_scale: float = 0.08,
+                 random_gray_scale: float = 0.2,
+                 hf_prob: float = 0.5,
+                 n_grid: int = 3,
+                 normalize: dict = imagenet_normalize
+        ):
+        super(PIRLCollateFunction, self).__init__()
+
+        if isinstance(input_size, tuple):
+            input_size_ = max(input_size)
+        else:
+            input_size_ = input_size
+
+        color_jitter = T.ColorJitter(
+            cj_bright, cj_contrast, cj_sat, cj_hue
+        )
+        
+        # Transform for transformed jigsaw image
+        transform = [
+            T.RandomHorizontalFlip(p=hf_prob),
+            T.RandomApply([color_jitter], p=cj_prob),
+            T.RandomGrayscale(p=random_gray_scale),
+            T.ToTensor()
+            ]
+
+        if normalize:
+            transform += [
+             T.Normalize(
+                mean=normalize['mean'],
+                std=normalize['std'])
+             ]
+
+        # Cropping and normalisation for untransformed image
+        self.no_augment = T.Compose([
+            T.RandomResizedCrop(size=input_size,
+                                scale=(min_scale,1.0)),
+            T.ToTensor(),
+            T.Normalize(
+                mean=normalize['mean'],
+                std=normalize['std'])
+        ])
+        self.jigsaw = Jigsaw(n_grid=n_grid,
+                             img_size=input_size_,
+                             crop_size=int(input_size_//n_grid),
+                             transform=T.Compose(transform))
+    
+    def forward(self, batch: List[tuple]):
+        """Overriding the BaseCollateFunction class's forward method because
+        for PIRL we need only one augmented batch, as opposed to both, which the
+        BaseCollateFunction creates."""
+        batch_size = len(batch)
+
+        # list of transformed images
+        img_transforms = [self.jigsaw(batch[i][0]).unsqueeze_(0)
+                      for i in range(batch_size)]
+        img = [self.no_augment(batch[i][0]).unsqueeze_(0)
+                for i in range(batch_size)]
+        # list of labels
+        labels = torch.LongTensor([item[1] for item in batch])
+        # list of filenames
+        fnames = [item[2] for item in batch]
+
+        # tuple of transforms
+        transforms = (
+            torch.cat(img, 0),
+            torch.cat(img_transforms, 0)
+        )
+
+        return transforms, labels, fnames
+
+
+class MSNCollateFunction(MultiViewCollateFunction):
+    """Implements the transformations for MSN [0].
+
+    Generates a set of random and focal views for each input image. The generated output
+    is (views, target, filenames) where views is list with the following entries:
+    [random_views_0, random_views_1, ..., focal_views_0, focal_views_1, ...].
+
+    - [0]: Masked Siamese Networks, 2022: https://arxiv.org/abs/2204.07141
+
+    Attributes:
+        random_size:
+            Size of the random image views in pixels.
+        focal_size:
+            Size of the focal image views in pixels.
+        random_views:
+            Number of random views to generate.
+        focal_views:
+            Number of focal views to generate.
+        random_crop_scale:
+            Minimum and maximum size of the randomized crops for the relative to random_size.
+        focal_crop_scale:
+            Minimum and maximum size of the randomized crops relative to focal_size.
+        cj_prob:
+            Probability that color jittering is applied.
+        cj_strength:
+            Strength of the color jitter.
+        gaussian_blur:
+            Probability of Gaussian blur.
+        kernel_size:
+            Sigma of gaussian blur is kernel_size * input_size.
+        random_gray_scale:
+            Probability of conversion to grayscale.
+        hf_prob:
+            Probability that horizontal flip is applied.
+        vf_prob:
+            Probability that vertical flip is applied.
+        normalize:
+            Dictionary with 'mean' and 'std' for torchvision.transforms.Normalize.
+    """
+    def __init__(
+        self,
+        random_size: int = 224,
+        focal_size: int = 96,
+        random_views: int = 2,
+        focal_views: int = 10,
+        random_crop_scale: Tuple[float, float] = (0.3, 1.0),
+        focal_crop_scale: Tuple[float, float] = (0.05, 0.3),
+        cj_prob: float = 0.8,
+        cj_strength: float = 1.0,
+        gaussian_blur: float = 0.5,
+        kernel_size: float = 0.1,
+        random_gray_scale: float = 0.2,
+        hf_prob: float = 0.5,
+        vf_prob: float = 0.0,
+        normalize: dict = imagenet_normalize
+    ) -> None:
+        color_jitter = T.ColorJitter(
+            brightness=0.8 * cj_strength,
+            contrast=0.8 * cj_strength,
+            saturation=0.8 * cj_strength,
+            hue=0.2 * cj_strength,
+        )
+        transform = T.Compose([
+            T.RandomResizedCrop(size=random_size, scale=random_crop_scale),
+            T.RandomHorizontalFlip(p=hf_prob),
+            T.RandomVerticalFlip(p=vf_prob),
+            T.RandomApply([color_jitter], p=cj_prob),
+            T.RandomGrayscale(p=random_gray_scale),
+            GaussianBlur(kernel_size=kernel_size, prob=gaussian_blur),
+            T.ToTensor(),
+            T.Normalize(mean=normalize['mean'], std=normalize['std']),
+        ])
+        focal_transform = T.Compose([
+            T.RandomResizedCrop(size=focal_size, scale=focal_crop_scale),
+            T.RandomHorizontalFlip(p=hf_prob),
+            T.RandomVerticalFlip(p=vf_prob),
+            T.RandomApply([color_jitter], p=cj_prob),
+            T.RandomGrayscale(p=random_gray_scale),
+            GaussianBlur(kernel_size=kernel_size, prob=gaussian_blur),
+            T.ToTensor(),
+            T.Normalize(mean=normalize['mean'], std=normalize['std']),
+        ])
+        transforms = [transform] * random_views
+        transforms += [focal_transform] * focal_views
+        super().__init__(transforms=transforms)
