@@ -10,6 +10,7 @@ can be combined with any lightly model.
 
 from lightly.models.modules.heads import BarlowTwinsProjectionHead
 from lightly.models.modules.heads import BYOLProjectionHead
+from lightly.models.modules.heads import BYOLPredictionHead
 from lightly.models.modules.heads import DINOProjectionHead
 from lightly.models.modules.heads import MoCoProjectionHead
 from lightly.models.modules.heads import NNCLRProjectionHead
@@ -21,3 +22,9 @@ from lightly.models.modules.heads import SwaVProjectionHead
 from lightly.models.modules.heads import SwaVPrototypes
 from lightly.models.modules.nn_memory_bank import NNMemoryBankModule
 
+from lightly import _torchvision_vit_available
+if _torchvision_vit_available:
+    # Requires torchvision >=0.12
+    from lightly.models.modules.masked_autoencoder import MAEBackbone
+    from lightly.models.modules.masked_autoencoder import MAEDecoder
+    from lightly.models.modules.masked_autoencoder import MAEEncoder
